from urllib.parse import urlparse

from .http import State
from typing import Optional
from .exceptions import Error
from .models import (
  InstagramUser,
  InstagramStory,
  InstagramHighlight,
  InstagramMedia
)

class Instagram:
  def __init__(self: "Instagram", csrf: str, session_id: str):
    self.session = State()
    self.headers = {
      "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 12_3_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 105.0.0.11.118 (iPhone11,8; iOS 12_3_1; en_US; en-US; scale=2.00; 828x1792; 165586599)",
      "Cookie": f"csrftoken={csrf}; sessionid={session_id}",
    }

  async def get_user(self: "Instagram", username: str):
    """
    Get user information by username.

    Parameters
    ----------
    username: :class:`str`
      The username of the user to fetch the info.
    
    Returns
    -------
    :class:`InstagramUser`
      An InstagramUser object with the user info.
    """
    data = await self.session.request(
      "GET",
      f"https://i.instagram.com/api/v1/users/{username}/usernameinfo",
      headers=self.headers,
    )

    if not data.user:
      raise Error(f"Can't find an user with the username `{username}`.")

    return InstagramUser(**data.user)

  async def get_story(self: "Instagram", username: str, amount: Optional[int] = None):
    """
    Get the stories of a user by username.

    Parameters
    ----------
    username: :class:`str`
      The username of the user to fetch the stories.
    amount: Optional[:class:`int`]
      The amount of stories to fetch. If the amount is not given all stories will be fetched.
    
    Returns
    -------
    :class:`List[InstagramStory]`
      A list of InstagramStory objects with the user stories.
    """
    user_id = (await self.get_user(username)).pk
    data = (await self.session.request(
      "GET",
      f"https://i.instagram.com/api/v1/feed/user/{user_id}/story/",
      headers=self.headers,
    )).get("reel", {})

    stories = []
    for story in data.get("items", []):
      if "video_versions" in story:
        story.video_url = sorted(
          story.video_versions,
          key=lambda x: x.height * x.width
        )[-1].url
      
      if "image_versions2" in story:
        story.image_url = sorted(
          story.image_versions2.candidates,
          key=lambda x: x.height * x.width,
        )[-1].url

      stories.append(story)

    if amount:
      stories = stories[:amount]

    return [InstagramStory(**s) for s in stories]

  async def get_highlights(self: "Instagram", username: str, amount: Optional[int] = None):
    """
    Get the highlights of a user by username.

    Parameters
    ----------
    username: :class:`str`
      The username of the user to fetch the highlights.
    amount: Optional[:class:`int`]
      The amount of highlights to fetch. If the amount is not given all highlights will be fetched.
    
    Returns
    -------
    :class:`List[InstagramHighlight]`
      A list of InstagramHighlight objects with the user highlights.
    """
    user_id = (await self.get_user(username)).pk
    data = await self.session.request(
      "GET",
      f"https://i.instagram.com/api/v1/highlights/{user_id}/highlights_tray/",
      headers=self.headers,
    )
    highlights = []
    for highlight in data.get("tray", []):
      highlight.id = highlight.id.split(":")[1]
      highlight.cover_media = highlight.cover_media.cropped_image_version.url

      highlights.append(highlight)

    if amount:
      highlights = highlights[:amount]

    return [InstagramHighlight(**h) for h in highlights]

  async def get_post(self: "Instagram", url: str, amount: Optional[int] = None):
    """
    Get the info about a post by url.

    Parameters
    ----------
    url: :class:`str`
      The url of the post to fetch it.
    amount: Optional[:class:`int`]
      The amount of images/videos to fetch. If the amount is not given all will be fetched.
    
    Returns
    -------
    :class:`List[InstagramMedia]`
      A list of InstagramMedia objects with the post images/videos.
    """
    char = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
    code = [p for p in (urlparse(url).path).split("/") if p][1]
    media_id = sum(char.index(x) * (len(char) ** i) for i, x in enumerate(reversed(code)))

    data = await self.session.request(
      "GET",
      f"https://i.instagram.com/api/v1/media/{media_id}/info",
      headers=self.headers,
    )

    medias = []
    for media in data.get("items", []):
      if "video_versions" in media:
        media.video_url = sorted(
          media.video_versions,
          key=lambda x: x.height * x.width,
        )[-1].url

      if "image_versions2" in media:
        urls = [media_url.url for media_url in media.image_versions2.candidates]
        if amount:
          urls = urls[:amount]

        media.image_urls = urls
        media.thumbnail_url = sorted(
          media.image_versions2.candidates,
          key=lambda x: x.height * x.width,
        )[-1].url

      medias.append(media)

    return [InstagramMedia(**m) for m in medias]


from re import (
  findall,
  match
)

from .http import State
from .exceptions import Error
from .models import BraveImages

class Brave:
  def __init__(self: "Brave"):
    self.session = State()
  
  async def get_images(self: "Brave", query: str, safe: bool = True):
    """
    Get images from Brave.

    Parameters
    ----------
    query: :class:`str`
      The query to search for.
    safe: :class:`bool`
      Whether to enable or disable safe search. Default is `True`.
    
    Returns
    -------
    :class:`BraveImages`
      A BraveImages object with the search results.
    """
    data = await self.session.request(
      "GET",
      "https://search.brave.com/images",
      params={
        "q": query,
        "safesearch": "strict" if safe else "off",
      }
    )
    imgs = findall(r'<img[^>]+src="([^">]+)"', data)
    if not (
      r := list(
        filter(
          lambda img: match(r"^https://imgs.search.brave.com/", img) and
          '32:32' not in img, imgs
        )
      )
    ):
      raise Error(f"No images were found for the query `{query}`.")

    return BraveImages(
      query=query,
      images=r,
    )


from datetime import datetime

from typing import (
  Optional,
  List
)
from pydantic import (
  BaseModel,
  HttpUrl
)

class BraveImages(BaseModel):
  query: str
  images: List[HttpUrl]

class TikTokBioLinks(BaseModel):
  link: str

class TikTokStats(BaseModel):
  followingCount: int
  followerCount: int
  heartCount: int
  videoCount: int

class TikTokUser(BaseModel):
  id: int
  uniqueId: str
  nickname: str
  signature: str
  avatarMedium: HttpUrl
  verified: bool
  privateAccount: bool
  stats: TikTokStats

class BioLinks(BaseModel):
  link_id: int
  url: str
  title: Optional[str] = None
  is_pinned: Optional[bool] = None

class InstagramUser(BaseModel):
  pk: int
  username: str
  full_name: str
  is_private: bool
  is_verified: bool
  media_count: int
  follower_count: int
  following_count: int
  is_business: bool
  profile_pic_url: HttpUrl
  biography: Optional[str] = None
  account_type: Optional[int] = None
  external_url: Optional[str] = None
  bio_links: List[BioLinks] = []

class InstagramStoryUser(BaseModel):
  pk: int
  username: Optional[str] = None
  full_name: Optional[str] = None
  profile_pic_url: Optional[HttpUrl] = None
  is_private: Optional[bool] = None

class InstagramStory(BaseModel):
  id: int
  media_type: int
  taken_at: datetime
  user: InstagramStoryUser
  image_url: Optional[HttpUrl] = None
  video_url: Optional[HttpUrl] = None
  video_duration: Optional[float] = 0.0
  thumnail_url: Optional[HttpUrl] = None

class InstagramHighlight(BaseModel):
  id: int
  title: str
  created_at: datetime
  is_pinned_highlight: bool
  media_count: int
  cover_media: str
  user: InstagramStoryUser

class InstagramMedia(BaseModel):
  id: int
  code: str
  media_type: int
  taken_at: datetime
  like_count: int
  comment_count: int
  play_count: Optional[int] = None
  title: Optional[str] = None
  user: InstagramStoryUser
  image_urls: Optional[List[HttpUrl]] = None
  thumbnail_url: Optional[HttpUrl] = None
  video_url: Optional[HttpUrl] = None
  video_duration: Optional[float] = 0.0


from re import search
from orjson import loads
from munch import munchify

from .http import State
from .exceptions import Error
from .models import (
  TikTokUser,
  TikTokStats
)

class TikTok:
  def __init__(self: "TikTok"):
    self.session = State()
  
  async def get_user(self: "TikTok", username: str):
    """
    Get use information by username.

    Parameters
    ----------
    username: :class:`str`
      The username of the user to fetch the info.
    
    Returns
    -------
    :class:`TikTokUser`
      A TikTokUser object with the user info.
    """
    data = await self.session.request(
      "GET",
      f"https://www.tiktok.com/@{username}",
    )
    result = search(r'<script id="__UNIVERSAL_DATA_FOR_REHYDRATION__"[^>]*>(.*?)</script>', data)
    raw = loads(result.group(1))["__DEFAULT_SCOPE__"]["webapp.user-detail"]
    loaded = munchify(raw)

    if loaded.statusCode == 10221:
      raise Error(f"Can't find an user with the username `@{username}`.")

    stats = TikTokStats(**loaded.userInfo.stats)
    loaded.userInfo.user.stats = stats
    return TikTokUser(**loaded.userInfo.user)